@@ -11,7 +11,7 @@
 # This file implements some common TCL routines used for regression
 # testing the SQLite library
 #
-# $Id: tester.tcl,v 1.65 2006/06/23 08:05:38 danielk1977 Exp $
+# $Id: tester.tcl,v 1.69 2006/10/04 11:55:50 drh Exp $
 
 # Make sure tclsqlite3 was compiled correctly.  Abort now with an
 # error message if not.
@@ -75,7 +75,6 @@ if {[info exists nTest]} return
 #
 set nErr 0
 set nTest 0
-set nProb 0
 set skip_test 0
 set failList {}
 set maxErr 1000
@@ -139,7 +138,7 @@ proc finish_test {} {
   finalize_testing
 }
 proc finalize_testing {} {
-  global nTest nErr nProb sqlite_open_file_count
+  global nTest nErr sqlite_open_file_count
   if {$nErr==0} memleak_check
 
   catch {db close}
@@ -149,7 +148,6 @@ proc finalize_testing {} {
   catch {
     pp_check_for_leaks
   }
-breakpoint
   sqlite3 db {}
   # sqlite3_clear_tsd_memdebug
   db close
@@ -162,15 +160,23 @@ breakpoint
   incr nTest
   puts "$nErr errors out of $nTest tests"
   puts "Failures on these tests: $::failList"
-  if {$nProb>0} {
-    puts "$nProb probabilistic tests also failed, but this does"
-    puts "not necessarily indicate a malfunction."
+  if {$nErr>0 && ![working_64bit_int]} {
+    puts "******************************************************************"
+    puts "N.B.:  The version of TCL that you used to build this test harness"
+    puts "is defective in that it does not support 64-bit integers.  Some or"
+    puts "all of the test failures above might be a result from this defect"
+    puts "in your TCL build."
+    puts "******************************************************************"
   }
-  if 0 {
   if {$sqlite_open_file_count} {
     puts "$sqlite_open_file_count files were left open"
     incr nErr
   }
+  foreach f [glob -nocomplain test.db-*-journal] {
+    file delete -force $f
+  }
+  foreach f [glob -nocomplain test.db-mj*] {
+    file delete -force $f
   }
   exit [expr {$nErr>0}]
 }
@@ -317,6 +323,7 @@ proc crashsql {crashdelay crashfile sql} {
 #     -tclbody          TCL script to run with IO error simulation.
 #     -sqlbody          TCL script to run with IO error simulation.
 #     -exclude          List of 'N' values not to test.
+#     -erc              Use extended result codes
 #     -start            Value of 'N' to begin with (default 1)
 #
 #     -cksum            Boolean. If true, test that the database does
@@ -326,7 +333,7 @@ proc do_ioerr_test {testname args} {
 
   set ::ioerropts(-start) 1
   set ::ioerropts(-cksum) 0
-
+  set ::ioerropts(-erc) 0
   array set ::ioerropts $args
 
   set ::go 1
@@ -347,6 +354,7 @@ proc do_ioerr_test {testname args} {
       catch {file delete -force test2.db}
       catch {file delete -force test2.db-journal}
       set ::DB [sqlite3 db test.db; sqlite3_connection_pointer db]
+      sqlite3_extended_result_codes $::DB $::ioerropts(-erc)
       if {[info exists ::ioerropts(-tclprep)]} {
         eval $::ioerropts(-tclprep)
       }
@@ -381,6 +389,19 @@ proc do_ioerr_test {testname args} {
     # a result of the script, the Nth will fail.
     do_test $testname.$n.3 {
       set r [catch $::ioerrorbody msg]
+      # puts rc=[sqlite3_errcode $::DB]
+      set rc [sqlite3_errcode $::DB]
+      if {$::ioerropts(-erc)} {
+        # In extended result mode, all IOERRs are qualified 
+        if {[regexp {^SQLITE_IOERR} $rc] && ![regexp {IOERR\+\d} $rc]} {
+          return $rc
+        }
+      } else {
+        # Not in extended result mode, no errors are qualified
+        if {[regexp {\+\d} $rc]} {
+          return $rc
+        }
+      }
       set ::go [expr {$::sqlite_io_error_pending<=0}]
       set s [expr $::sqlite_io_error_hit==0]
       set ::sqlite_io_error_hit 0
